/*
** Copyright (C) Red Hat, Inc. 2010
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; see the file COPYING. If not, write to the
** Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
** MA 02139, USA.
*/

/*
 * This program is used so we can authenticate against PAM from a non-root
 * python program.
 *
 * Author: Chris Feist <cfeist@redhat.com>
 */

#include <python2.6/Python.h>
#include <sasl/sasl.h>
#include <string.h>

static PyObject * lucipam_checkauth(PyObject *self, PyObject *args);
int authenticate(const char* username, const char* password);
static int sasl_getopts_callback(void* context, const char *plugin_name,
                      const char *option, const char **result,
                      unsigned int *len);

sasl_callback_t callbacks[] = {
    { SASL_CB_GETOPT, (int (*)()) sasl_getopts_callback, NULL }
};

int sasl_getopts_callback(  void *context, const char *plugin_name,
                const char *option, const char **result, unsigned *len)
{

    static const char authd_option[] = "pwcheck_method";
    static const char authd_result[] = "saslauthd";
    static const char authd_version_option[] = "saslauthd_version";
    static const char authd_version_result[] = "2";

    if (result) {
        *result = 0;
        if (!strcmp(option, authd_option))
            *result = authd_result;
        else if (!strcmp(option, authd_version_option))
            *result = authd_version_result;
    }

    if (len)
        *len = 0;

    return SASL_OK;
}


static PyMethodDef LucipamMethods[] = {
 {"checkauth", lucipam_checkauth, METH_VARARGS,
  "Authenticate against PAM."},
 {NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC initlucipam(void)
{
    (void) Py_InitModule("lucipam", LucipamMethods);
}

static PyObject * lucipam_checkauth(PyObject *self, PyObject *args)
{
    const char *username;
    const char *password;

    if (!PyArg_ParseTuple(args, "ss", &username, &password))
	return NULL;

    if (authenticate(username,password))
        return Py_BuildValue("i",1);
    else
        return Py_BuildValue("i",0);
}    

int authenticate(const char *username, const char *password)
{
  int ret;
  sasl_conn_t *conn;

  ret = sasl_server_init(callbacks, "luci");
  if (ret != SASL_OK) {
      return 0;
  }
  
  ret = sasl_server_new("luci", // servicename
      NULL,           // hostname
      NULL,           // realm
      NULL,           // local ip:port
      NULL,           // remote ip:port
      NULL,
      0,                      // connection flags
      &conn);
  if (ret != SASL_OK)
  {
      return 0;
  }

  if (sasl_checkpass(conn, username, strlen(username), password,
                     strlen(password)) == SASL_OK)
      return 1;
  else
      return 0;
}
